import 'dart:convert';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:vitalapp/database/db.dart';
import 'package:vitalapp/database/entities/defines.dart';
import 'package:vitalapp/database/entities/diagnosis.dart';
import 'package:vitalapp/database/entities/patient.dart';
import 'package:vitalapp/global.dart';
import 'package:vitalapp/managers/interfaces/diagnosis.dart';
import 'package:vitalapp/managers/interfaces/models/diagnosis_aggregation_record_model.dart';
import 'package:vitalapp/rpc.dart';
import 'package:vitalapp/store/store.dart';
import 'package:fis_common/logger/logger.dart';

class DiagnosisManager implements IDiagnosisManager {
  DiagnosisManager();

  @override
  Future<PageCollection<DiagnosisAggregationRecordModel>?>
      getDiagnosisAggregationPageAsync(
          String patientCode, int pageIndex, int pageSize) async {
    PageCollection<DiagnosisAggregationRecordModel> diagnosisAggregationRecord =
        PageCollection<DiagnosisAggregationRecordModel>();
    try {
      var request = DiagnosisPageRequest(
        token: Store.user.token,
        pageIndex: pageIndex,
        pageSize: pageSize,
        patientCode: patientCode,
      );
      diagnosisAggregationRecord.pageData = [];
      var result =
          await rpc.diagnosis.getDiagnosisAggregationPageAsync(request);
      diagnosisAggregationRecord.pageIndex = result.pageIndex;
      diagnosisAggregationRecord.pageSize = result.pageSize;
      diagnosisAggregationRecord.dataCount = result.dataCount;
      for (var element in result.pageData!) {
        var record = DiagnosisAggregationRecordModel.fromJson(element.toJson());
        record.isExistLocalData = false;
        diagnosisAggregationRecord.pageData!.add(record);
      }
    } catch (e) {}
    return diagnosisAggregationRecord;
  }

  @override
  Future<List<DiagnosisAggregationRecordModel>?> getListByPatientCode(
      String patientCode) async {
    try {
      List<DiagnosisAggregationRecordModel> records =
          <DiagnosisAggregationRecordModel>[];
      List<DiagnosisEntity> localRecords = [];
      localRecords = await db.repositories.diagnosis
          .getNotUploadedListByPatientCode(patientCode);
      if (!kIsOnline) {
        localRecords =
            await db.repositories.diagnosis.getListByPatientCode(patientCode);
      }
      var currentPatient = Store.user.currentSelectPatientInfo!;
      for (var element in localRecords) {
        Map<String, dynamic> data = jsonDecode(element.dataJson);
        List<DiagnosisAggregationData> list = [];
        for (var key in data.keys) {
          list.add(DiagnosisAggregationData(
              key: key, diagnosisData: jsonEncode(data[key])));
        }
        records.add(DiagnosisAggregationRecordModel(
          patientCode: element.patientCode,
          patientName: currentPatient.patientName,
          doctorName: Store.user.displayName,
          diagnosisTime: element.createTime,
          diagnosisAggregationData: list,
          isExistLocalData: element.syncState != OfflineDataSyncState.success,
        ));
      }
      return records;
    } catch (e) {}
    return null;
  }

  /// 创建健康检测
  @override
  Future<bool> submitDiagnosisAsync(SubmitDiagnosisRequest request) async {
    request.token = Store.user.token;
    print(jsonEncode(request.toJson()));
    var result = await rpc.diagnosis.submitDiagnosisAsync(request);

    return result;
  }

  /// 创建健康检测（在线）
  @override
  Future<bool> syncPatientAndDiagnosisData(
      SyncPatientAndDiagnosisDataRequest request) async {
    try {
      var result = await rpc.diagnosis.syncPatientAndDiagnosisDataAsync(
        request,
      );
      if (result) {
        final code = request.cardNo!;
        var existEntity = await _checkPatientExist(code);

        if (existEntity != null) {
          existEntity.syncState = OfflineDataSyncState.success;
          await db.repositories.patient.update(existEntity);
        }
      }
      return result;
    } catch (e) {
      logger.e(
          "syncPatientAndDiagnosisData - Check patient exist error. Code: ${request.cardNo}.",
          e);
      return false;
    }
  }

  Future<PatientEntity?> _checkPatientExist(String code) async {
    try {
      final entity = await db.repositories.patient.singleByCode(code);
      return entity;
    } catch (e) {
      logger.e(
          "PatientServiceMock - Check patient exist error. Code: $code.", e);
    }
    return null;
  }
}
