import 'package:flutter/material.dart';
import 'package:vitalapp/architecture/network_connectivity.dart';
import 'package:vitalapp/global.dart';
import 'package:vitalapp/pages/help/help_dialog.dart';

class HeaderStatusBar extends StatelessWidget {
  const HeaderStatusBar({super.key});

  @override
  Widget build(BuildContext context) {
    return Row(
      mainAxisSize: MainAxisSize.min,
      children: [
        _NetStatusWidget(),
        const SizedBox(width: 2),
        IconButton(
          onPressed: () {
            HelpDialog.show();
          },
          icon: const Icon(Icons.help, color: Colors.white),
        ),
      ],
    );
  }
}

class _NetStatusWidget extends StatefulWidget {
  @override
  State<StatefulWidget> createState() => _NetStatusWidgetState();
}

class _NetStatusWidgetState extends State<_NetStatusWidget> {
  @override
  void initState() {
    super.initState();
    WidgetsBinding.instance.addPostFrameCallback((timeStamp) {
      netChecker.onlineChangedEvent.addListener(_onlineChanged);
    });
  }

  @override
  void dispose() {
    netChecker.onlineChangedEvent.removeListener(_onlineChanged);
    super.dispose();
  }

  void _onlineChanged(_, e) {
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    if (kIsOnline) {
      return const Tooltip(
        triggerMode: TooltipTriggerMode.tap,
        message: "网络已连接",
        child: Icon(Icons.wifi, color: Colors.green),
      );
    } else {
      return const Tooltip(
        triggerMode: TooltipTriggerMode.tap,
        message: "网络已断开",
        child: Icon(Icons.wifi_off_outlined, color: Colors.red),
      );
    }
  }
}
