import 'package:fis_jsonrpc/http_pool.dart';
import 'package:get/get.dart';
import 'package:vitalapp/architecture/defines.dart';
import 'package:vitalapp/architecture/utils/prompt_box.dart';
import 'package:vitalapp/managers/interfaces/account.dart';
import 'package:vitalapp/rpc.dart';
import 'package:vitalapp/store/store.dart';
import 'package:fis_common/logger/logger.dart';

import 'state.dart';

class ServerSettingController extends FControllerBase {
  final state = ServerSettingState();

  @override
  void onReady() {
    _load();
    super.onReady();
  }

  /// 保存配置
  Future<void> save() async {
    RegExp regExp = RegExp(r'\s'); // 匹配空格的正则表达式
    bool hasSpace = regExp.hasMatch(state.host);
    if (hasSpace) {
      PromptBox.toast("请输入正确的服务器地址");
      return;
    }
    final uri = Uri(
      scheme: state.enableSSL ? "https" : "http",
      host: state.host,
      port: state.port,
    );
    final fullAddress = uri.toString();
    if (fullAddress == Store.app.serverUrl) {
      PromptBox.toast("未进行变更服务器地址");
      return;
    }
    Store.app.serverUrl = fullAddress;
    var checkResult = await _checkServerValidation();
    if (!checkResult) {
      PromptBox.toast("服务器不存在");
      return;
    }
    setBusy("正在保存");
    try {
      await Store.persistent();
      PromptBox.toast("保存成功");
      final result = await Get.find<IAccountManager>().logout();
      if (result) {
        //  更新RPC地址
        rpc.setServerHost("${uri.host}:${uri.port}", uri.scheme == 'https');
        rpc.clearCache();
        Store.user.currentSelectPatientInfo = null;
        Get.offAllNamed("/login");
      }
    } catch (e) {
      logger.e("ServerSettingController save url error.", e);
      PromptBox.toast("保存失败");
    }
    cancelBusy();
  }

  void _load() async {
    final address = Store.app.serverUrl;
    final uri = Uri.parse(address);
    state.enableSSL = uri.scheme.toLowerCase() == "https";
    state.host = uri.host;
    state.port = uri.port;
  }

  /// 检验服务有效性
  Future<bool> _checkServerValidation() async {
    try {
      final host = Store.app.serverUrl;

      var response = await jrpcHttpPool.getClient(
        host,
        timeout: 3000,
        headers: {'content-type': "text/plain"},
      ).post(
        '/IVitalLoginService',
        data:
            '{"jsonrpc":"2.0","method":"CommonLoginAsync","params":["AnyAccount": "abc123","AnyCode": "","Platform": 1,"LoginSource": 1,"Password": "fb6414d24e3c347d46032b0496f1c4e4"],"id":1}',
      );
      if (response.data != null) {
        print(response.data);
        return true;
      }
    } catch (e) {
      print("Http unknown error:${e}");
    }
    return false;
  }
}
